import abc


class Template(abc.ABC):
    @classmethod
    @abc.abstractmethod
    def template(cls) -> str:
        pass

    @classmethod
    @abc.abstractmethod
    def fields(cls) -> list[str]:
        pass

    @classmethod
    def render(cls, **kwargs):
        # check fields
        cls._check_fields(**kwargs)
        return cls.template().format(**kwargs)

    @classmethod
    def _check_fields(cls, **kwargs):
        missing_fields = []
        redundant_fields = []
        error_msg = ""
        for field in cls.fields():
            if field not in kwargs:
                missing_fields.append(field)
        for field in kwargs:
            if field not in cls.fields():
                redundant_fields.append(field)
        if len(missing_fields) > 0:
            error_msg += f"Missing fields: {missing_fields}\n"
        if len(redundant_fields) > 0:
            error_msg += f"Redundant fields: {redundant_fields}\n"
        if len(error_msg) > 0:
            raise ValueError(error_msg)


__all__ = ['Template']



from ..base_template import Template

fields = [
    'project_name',
    'author',
]

template = """from setuptools import setup, find_packages


def read_requirements():
    with open('requirements.txt', 'r') as f:
        requirements = f.readlines()
    return [req.strip() for req in requirements if req.strip() and not req.startswith('#')]


setup(
    name='{project_name}',
    version='0.0.1',
    packages=find_packages(exclude=['test', 'test.*']),
    include_package_data=True,
    install_requires=read_requirements(),
    author='{author}',
    author_email='',
    description='',
    long_description=open('./README.md').read(),
    long_description_content_type='text/markdown',
)

# To build:
# python setup.py sdist bdist_wheel
# To upload to pypi:
# twine upload dist/*
"""


class SetupPyTemplate(Template):
    @classmethod
    def template(cls) -> str:
        return template

    @classmethod
    def fields(cls) -> list[str]:
        return fields


__all__ = [SetupPyTemplate]



from .base_template import Template

fields = ['project_name']

template = """# {project_name}
"""


class READMEMarkdownTemplate(Template):
    @classmethod
    def template(cls) -> str:
        return template

    @classmethod
    def fields(cls) -> list[str]:
        return fields


__all__ = [READMEMarkdownTemplate]



from .launcher import *

ide_dict = {
    "pycharm": PyCharmLauncher,
    "vscode": VSCodeLauncher,
    "webstorm": WebStormLauncher,
    "idea": IntelliJIDEALauncher,
}


def launch_ide(ide: str, project_path: str):
    """
    Launch the specified IDE with the specified project path

    :param ide: (str) the name of the IDE to be launched
    :param project_path: (str) the path of the project to be opened
    :return: (None)
    """
    ide = ide.lower()
    if ide not in ide_dict:
        raise ValueError(f"IDE {ide} is not supported. Supported IDEs are {list(ide_dict.keys())}")

    ide_launcher = ide_dict[ide]()
    ide_launcher.launch(project_path=project_path)


__all__ = ["launch_ide"]



import shutil
import pathlib
import argparse
from typing import Optional
from datetime import datetime

from .template import READMEMarkdownTemplate, LicenceTemplate
from .template.bolt_py import GitIgnorePyTemplate, SetupPyTemplate


def create_python_project(
        proj_name: str,
        author: str,
        parent_dir: str = ".",
        is_pypi_proj: bool = False,
        is_git_proj: bool = False,
):
    """Create a Template Python project

    Args:
        proj_name: (str) name of the project
        author: (str) author of the project
        parent_dir: (str) parent directory where the project will be created. Defaults to the current directory.
        is_pypi_proj: (bool) specify if it's a PyPI project. If set, appropriate setup files will be generated.
        is_git_proj: (bool) specify if it's a Git project. If set, a git repository will be initialized.

    Returns:
        None
    """
    # 创建项目目录, 若已存在则报错
    print(f"new version!!")
    proj_path = pathlib.Path(parent_dir) / proj_name
    proj_path.mkdir(parents=True, exist_ok=False)  # 创建项目文件夹

    try:
        # 使用模板文件创建和写入文件
        (proj_path / 'README.md').write_text(READMEMarkdownTemplate.render(project_name=proj_name))
        (proj_path / 'requirements.txt').touch()

        # 如果 is_pypi_proj 为 True
        if is_pypi_proj:
            # 从模板中创建 PyPI 项目所需文件
            (proj_path / 'setup.py').write_text(SetupPyTemplate.render(project_name=proj_name, author=author))
            (proj_path / 'MANIFEST.in').touch()

            (proj_path / 'LICENSE').write_text(LicenceTemplate.render(author=author, year=datetime.now().year))

            pkg_path = proj_path / proj_name
            pkg_path.mkdir(parents=True, exist_ok=True)

            (pkg_path / '__init__.py').touch()

        # 如果 is_git_proj 为 True
        if is_git_proj:
            # 创建 Git 项目所需文件
            (proj_path / '.gitignore').write_text(GitIgnorePyTemplate.render())
    except Exception as e:
        # 如果出错, 则删除项目目录
        proj_path.rmdir()
        # use rmtree instead
        shutil.rmtree(proj_path)
        raise e


def query_user_for_input(prompt: str, default: Optional[str] = None) -> str:
    """
    Query the user for input with a default value.

    :param prompt: The prompt to display to the user.
    :param default: The default value to use if the user doesn't provide input.
    :return: The user's input or the default value.
    """
    user_input = input(f"{prompt} [{'default: ' + default if default else ''}] > ").strip()
    return user_input or default


def main():
    parser = argparse.ArgumentParser(description='Create a Template Python project.')

    parser.add_argument('-n', metavar='PROJECT_NAME', help='Name of the project.')
    parser.add_argument('-p', '--parent-dir', default=None, help='Parent directory where the project will be created. Defaults to the current directory.')
    parser.add_argument('--pypi', action='store_true', default=None, help='Specify if it\'s a PyPI project. If set, appropriate setup files will be generated.')
    parser.add_argument('--git', action='store_true', default=None, help='Specify if it\'s a Git project. If set, a git repository will be initialized.')
    parser.add_argument("--author", metavar="AUTHOR", default=None, help="The author of the project.")

    args = parser.parse_args()

    if not args.n:
        args.n = query_user_for_input("Please enter the project name:")

    if not args.author:
        args.author = query_user_for_input("Please enter the author name:")

    if args.parent_dir is None:
        args.parent_dir = query_user_for_input("Please enter the parent directory for the project:", ".")

    if args.pypi is None:
        args.pypi = query_user_for_input("Is it a PyPI project? (y/n):", "n").lower() == 'y'

    if args.git is None:
        args.git = query_user_for_input("Is it a Git project? (y/n):", "n").lower() == 'y'

    create_python_project(
        proj_name=args.n,
        is_git_proj=args.git,
        is_pypi_proj=args.pypi,
        parent_dir=args.parent_dir,
        author=args.author,
    )



import os
import abc
import pathlib
import traceback
import subprocess
from typing import Union


class BaseIDELauncher(abc.ABC):
    """
    Base class for IDE launcher

    IDELauncher is designed to help users launch their IDEs from command line.

    User interface:

    `launch(project_path: Union[str, pathlib.Path])`: launch the IDE with the specified project path
    """
    @property
    @abc.abstractmethod
    def ide_name(self) -> str:
        """user-friendly name of the IDE"""
        raise NotImplementedError()

    @property
    @abc.abstractmethod
    def ide_home_env_var_name(self) -> str:
        """the environment variable name of the IDE home directory, this should be a existing os environment variable"""
        raise NotImplementedError()

    def launch(self, project_path: Union[str, pathlib.Path]):
        """
        Launch the IDE with the specified project path

        :param project_path: (Union[str, pathlib.Path]) the path of the project to be opened
        :return: (None)
        """
        project_path = pathlib.Path(project_path)

        # Check if project directory already exists
        if not project_path.exists():
            raise FileNotFoundError(f"Project directory {project_path} does not exist.")

        # Check whether the environment variable is set
        if self.ide_home_env_var_name is not None:
            if self.ide_home_env_var_name not in os.environ:
                raise EnvironmentError(
                    f"{self.ide_name} home environment variable {self.ide_home_env_var_name} is not set. Please set it first. It should be the path to the home directory of {self.ide_name}.")

        # Launch the IDE in a new process
        ide_home_dir = pathlib.Path(os.environ.get(self.ide_home_env_var_name))
        try:
            ide_exec_path = self.get_ide_exec_path(ide_home_dir, platform=os.name)
            subprocess.Popen([ide_exec_path, project_path.absolute().as_posix()], start_new_session=True)
        except Exception as e:
            raise RuntimeError(f"Failed to launch {self.ide_name}: {e}.\nTraceback: {traceback.format_exc()}.\nPlease submit an issue on GitHub: Thanks!") from e

    @abc.abstractmethod
    def get_ide_exec_path(self, ide_home_dir: pathlib.Path, platform: str) -> pathlib.Path:
        """find the executable path of the IDE, given the ide home directory and the platform"""
        raise NotImplementedError()


class PyCharmLauncher(BaseIDELauncher):
    @property
    def ide_name(self) -> str:
        return "PyCharm"

    @property
    def ide_home_env_var_name(self) -> str:
        return "PYCHARM_HOME"

    def get_ide_exec_path(self, ide_home_dir: pathlib.Path, platform: str) -> pathlib.Path:
        if platform == "nt":
            return ide_home_dir / "bin" / "pycharm64.exe"
        else:
            return ide_home_dir / "bin" / "pycharm.sh"


class VSCodeLauncher(BaseIDELauncher):
    @property
    def ide_name(self) -> str:
        return "VSCode"

    @property
    def ide_home_env_var_name(self) -> str:
        return "VSCODE_HOME"

    def get_ide_exec_path(self, ide_home_dir: pathlib.Path, platform: str) -> pathlib.Path:
        if platform == "nt":
            return ide_home_dir / "Code.exe"
        else:
            return ide_home_dir / "code"


class WebStormLauncher(BaseIDELauncher):
    @property
    def ide_name(self) -> str:
        return "WebStorm"

    @property
    def ide_home_env_var_name(self) -> str:
        return "WEBSTORM_HOME"

    def get_ide_exec_path(self, ide_home_dir: pathlib.Path, platform: str) -> pathlib.Path:
        if platform == "nt":
            return ide_home_dir / "bin" / "webstorm64.exe"
        else:
            return ide_home_dir / "bin" / "webstorm.sh"


class IntelliJIDEALauncher(BaseIDELauncher):
    @property
    def ide_name(self) -> str:
        return "IntelliJ IDEA"

    @property
    def ide_home_env_var_name(self) -> str:
        return "IDEA_HOME"

    def get_ide_exec_path(self, ide_home_dir: pathlib.Path, platform: str) -> pathlib.Path:
        if platform == "nt":
            return ide_home_dir / "bin" / "idea64.exe"
        else:
            return ide_home_dir / "bin" / "idea.sh"


__all__ = ["PyCharmLauncher", "VSCodeLauncher", "WebStormLauncher", "IntelliJIDEALauncher"]
